from typing import Any, Union

from telethon.sessions.memory import _SentFileType
from telethon.tl.types import InputPhoto, InputDocument

from .core import AlchemyCoreSession


class AlchemySQLiteCoreSession(AlchemyCoreSession):
    def set_update_state(self, entity_id: int, row: Any) -> None:
        with self.engine.begin() as conn:
            conn.execute(
                "INSERT OR REPLACE INTO {} ".format(self.UpdateState.__tablename__)
                + "VALUES (:session_id, :entity_id, :pts, :qts, :date, :seq, "
                "        :unread_count)",
                dict(
                    session_id=self.session_id,
                    entity_id=entity_id,
                    pts=row.pts,
                    qts=row.qts,
                    date=row.date.timestamp(),
                    seq=row.seq,
                    unread_count=row.unread_count,
                ),
            )

    def process_entities(self, tlo: Any) -> None:
        rows = self._entities_to_rows(tlo)
        if not rows:
            return

        with self.engine.begin() as conn:
            conn.execute(
                "INSERT OR REPLACE INTO {} ".format(self.Entity.__tablename__)
                + "VALUES (:session_id, :id, :hash, :username, :phone, :name)",
                [
                    dict(
                        session_id=self.session_id,
                        id=row[0],
                        hash=row[1],
                        username=row[2],
                        phone=row[3],
                        name=row[4],
                    )
                    for row in rows
                ],
            )

    def cache_file(
        self,
        md5_digest: str,
        file_size: int,
        instance: Union[InputDocument, InputPhoto],
    ) -> None:
        if not isinstance(instance, (InputDocument, InputPhoto)):
            raise TypeError("Cannot cache {} instance".format(type(instance)))

        t = self.SentFile.__table__
        values = dict(id=instance.id, hash=instance.access_hash)
        with self.engine.begin() as conn:
            conn.execute(
                "INSERT OR REPLACE INTO {} ".format(self.SentFile.__tablename__)
                + "VALUES (:session_id, :md5_digest, :type, :file_size)",
                dict(
                    session_id=self.session_id,
                    md5_digest=md5_digest,
                    type=_SentFileType.from_type(type(instance)).value,
                    file_size=file_size,
                    **values
                ),
            )



from typing import Any, Union

from sqlalchemy.dialects.postgresql import insert

from telethon.sessions.memory import _SentFileType
from telethon.tl.types import InputPhoto, InputDocument

from .core import AlchemyCoreSession


class AlchemyPostgresCoreSession(AlchemyCoreSession):
    def set_update_state(self, entity_id: int, row: Any) -> None:
        t = self.UpdateState.__table__
        values = dict(
            pts=row.pts,
            qts=row.qts,
            date=row.date.timestamp(),
            seq=row.seq,
            unread_count=row.unread_count,
        )
        with self.engine.begin() as conn:
            conn.execute(
                insert(t)
                .values(session_id=self.session_id, entity_id=entity_id, **values)
                .on_conflict_do_update(constraint=t.primary_key, set_=values)
            )

    def process_entities(self, tlo: Any) -> None:
        rows = self._entities_to_rows(tlo)
        if not rows:
            return

        t = self.Entity.__table__
        ins = insert(t)
        upsert = ins.on_conflict_do_update(
            constraint=t.primary_key,
            set_={
                "hash": ins.excluded.hash,
                "username": ins.excluded.username,
                "phone": ins.excluded.phone,
                "name": ins.excluded.name,
            },
        )
        with self.engine.begin() as conn:
            conn.execute(
                upsert,
                [
                    dict(
                        session_id=self.session_id,
                        id=row[0],
                        hash=row[1],
                        username=row[2],
                        phone=row[3],
                        name=row[4],
                    )
                    for row in rows
                ],
            )

    def cache_file(
        self,
        md5_digest: str,
        file_size: int,
        instance: Union[InputDocument, InputPhoto],
    ) -> None:
        if not isinstance(instance, (InputDocument, InputPhoto)):
            raise TypeError("Cannot cache {} instance".format(type(instance)))

        t = self.SentFile.__table__
        values = dict(id=instance.id, hash=instance.access_hash)
        with self.engine.begin() as conn:
            conn.execute(
                insert(t)
                .values(
                    session_id=self.session_id,
                    md5_digest=md5_digest,
                    type=_SentFileType.from_type(type(instance)).value,
                    file_size=file_size,
                    **values
                )
                .on_conflict_do_update(constraint=t.primary_key, set_=values)
            )



import os
import setuptools


if "requirements.txt" in os.listdir("."):
    with open("requirements.txt", encoding="utf-8") as r:
        requires = [i.strip() for i in r]  # Зависимости
else:
    requires = []


with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()


name = "telealchemy"
author = "alteralt"


setuptools.setup(
    name=name,
    version="1.0.3",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    author=author,
    include_package_data=True,
    python_requires=">=3.5",
    install_requires=requires,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: Russian",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    project_urls={"Source": "https://github.com/{}/{}".format(author, name)},
)



from typing import Any, Union

from sqlalchemy.dialects.mysql import insert

from telethon.sessions.memory import _SentFileType
from telethon.tl.types import InputPhoto, InputDocument

from .core import AlchemyCoreSession


class AlchemyMySQLCoreSession(AlchemyCoreSession):
    def set_update_state(self, entity_id: int, row: Any) -> None:
        t = self.UpdateState.__table__
        values = dict(
            pts=row.pts,
            qts=row.qts,
            date=row.date.timestamp(),
            seq=row.seq,
            unread_count=row.unread_count,
        )
        with self.engine.begin() as conn:
            conn.execute(
                insert(t)
                .values(session_id=self.session_id, entity_id=entity_id, **values)
                .on_duplicate_key_update(**values)
            )

    def process_entities(self, tlo: Any) -> None:
        rows = self._entities_to_rows(tlo)
        if not rows:
            return

        t = self.Entity.__table__
        with self.engine.begin() as conn:
            for row in rows:
                values = dict(hash=row[1], username=row[2], phone=row[3], name=row[4])
                conn.execute(
                    insert(t)
                    .values(session_id=self.session_id, id=row[0], **values)
                    .on_duplicate_key_update(**values)
                )

    def cache_file(
        self,
        md5_digest: str,
        file_size: int,
        instance: Union[InputDocument, InputPhoto],
    ) -> None:
        if not isinstance(instance, (InputDocument, InputPhoto)):
            raise TypeError("Cannot cache {} instance".format(type(instance)))

        t = self.SentFile.__table__
        values = {"id": instance.id, "hash": instance.access_hash}
        with self.engine.begin() as conn:
            conn.execute(
                insert(t)
                .values(
                    session_id=self.session_id,
                    md5_digest=md5_digest,
                    type=_SentFileType.from_type(type(instance)).value,
                    file_size=file_size,
                    **values
                )
                .on_duplicate_key_update(**values)
            )
